# -*- coding:utf-8 -*-
from .client import start_client
from .server import start_server


VERSION = '3.1.0'

AUTHOR = "cn"

AUTHOR_EMAIL = "308299269@qq.com"

URL = "https://www.github.com/ShichaoMa/custom_redis"
